
import sys
import unittest

from pathlib import Path

# sys.path[0] = str(Path(__file__).parent.parent.resolve())
#
# def test(filename):
#     if filename == '__main__':
#         unittest.main()

def foo():
    print("Foo")
